// ディスクの情報（総容量／空き容量など）を参照する。

//  Created by raktajino on Sat Dec. 4 2004.
//  Copyright (c) 2004 raktajino. All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
// OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
// IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
// INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
// NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package com.mac.raktajino.RDService;

import java.io.*;
import java.lang.Long;
import java.util.StringTokenizer;

public class DiskInformation {

	public	long	fFreeSpace = -1;
	public	long	fUsedSpace = -1;
	

	DiskInformation(File path)
	{
		fFreeSpace = fUsedSpace = -1;
		try {
			// dfコマンドを実行し、その実行結果を受け取る。
			String[]       cmdArray    = { "/bin/df", "-k", path.getAbsolutePath() };
			Process        process     = Runtime.getRuntime().exec(cmdArray);
		    BufferedReader inputReader = new BufferedReader(new InputStreamReader(process.getInputStream()));

		    String headerLine = inputReader.readLine();
			if(headerLine == null) {
				return;
			}
		    String resultLine = inputReader.readLine();
			if(resultLine == null) {
				return;
			}
		    process.waitFor();

			// dfコマンドの実行結果を解析し、ディスクの空き容量と使用済み容量を求める。
			StringTokenizer headerTokenizer = new StringTokenizer(headerLine, " ");
			StringTokenizer resultTokenizer = new StringTokenizer(resultLine, " ");
	
			while(headerTokenizer.hasMoreTokens() && resultTokenizer.hasMoreTokens()) {
				String headerTaken = headerTokenizer.nextToken();
				String resultTaken = resultTokenizer.nextToken();
				if(headerTaken.toLowerCase().startsWith("avail")) {
					fFreeSpace = Long.parseLong(resultTaken) * 1024L;
				}
				else if(headerTaken.toLowerCase().startsWith("used")) {
					fUsedSpace = Long.parseLong(resultTaken) * 1024L;
				}
				if(fFreeSpace >= 0 && fUsedSpace >= 0) {
					break;
				}
			}
		}
		catch(Exception e) {
			fFreeSpace = fUsedSpace = -1;
		}
	}
}
