// 東芝RDシリーズ DVD／HDDレコーダの“ネットdeダビング”機能を利用して、画像ファイルをコピーする。

//  Created by raktajino on Sun Nov. 21 2004.
//  Copyright (c) 2004 raktajino. All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
// OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
// IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
// INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
// NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package com.mac.raktajino.RDService;

import java.io.*;
import java.lang.*;
import java.util.*;

public class Main {

    public static final String	kProgramName	 = "RDService";			// このプログラムの名称
    public static final String	kProgramVersion	 = "1.0 beta 5";		// このプログラムのバージョン番号

	// 稼働プラットフォームを示すフラグ
	public static final	String	kOSName          = System.getProperty("os.name").toLowerCase();
	public static final	boolean	kPlatformMacOS   = (kOSName.indexOf("mac") >= 0);
	public static final	boolean	kPlatformWindows = (kOSName.indexOf("windows") >= 0);

	protected           int		fFTPControlPort	 = 21;					// FTPのコントロール・ポート番号
	protected           int		fFTPDataPortFrom = 0;					// FTPのデータ・ポートの開始番号
	protected           int		fFTPDataPortTo	 = 0;					// FTPのデータ・ポートの終了番号
	protected			int		fFTPBufferSize	 = 1024 * 1024;			// FTPデータ転送時のバッファ・サイズ
	protected           int		fNetBIOSNSPort   = 137;					// NETBIOS Name Serviceのポート番号
	protected           String	fDstDirectory    = ".";					// ファイル保存先のディレクトリ
	protected           String	fServiceName     = kProgramName;		// NETBIOSの公開名

	// --- メイン・プログラム
	
    public static void main(String args[])
	{
		// コマンド・ライン・オプションを処理する。
		Main main = new Main();
		main.processArgs(args);
	
		// ネットワーク・サービス・スレッドを起動する。
		main.start();
		
		// 標準入力ストリームが閉じられるまで待ち、閉じられたら終了する。
		while(true) {
			try {
				System.in.available();
				Thread.sleep(3000);			// 3秒ごとにポーリングする。
			}
			catch(InterruptedException e) {
			}
			catch(IOException e) {
				break;
			}
		}
		System.exit(0);
    }
	
	// --- ネットワーク・サービス・スレッドを起動する。
	
	protected void start()
	{
		// FTP Serviceを起動する。
		FTPService ftpService = new FTPService(fFTPControlPort, fFTPDataPortFrom, fFTPDataPortTo, fFTPBufferSize, fDstDirectory);
		ftpService.start();

		// Name Serviceを起動する。
		NameService nameService = new NameService(fNetBIOSNSPort, fServiceName);
		nameService.start();
	}

	// --- コマンドの引数を処理する。

	protected void processArgs(String args[])
	{
		boolean showSummary     = false;
		boolean showUsage       = false;
		boolean showHelp        = false;
		int     ftpDataPortFrom = fFTPDataPortFrom;
		int     ftpDataPortTo   = fFTPDataPortTo;
		int     ftpBufferSize   = fFTPBufferSize;
		String  dstDirectory    = fDstDirectory;
		String  serviceName     = fServiceName;
		
		// コマンドの引数を解析する。
		for(int n = 0; n < args.length; n++) {
			if(args[n].equals("-p")) {
				if(n + 1 >= args.length || args[n + 1].startsWith("-")) {
					System.err.println("Missing a port number : " + args[n]);
					showUsage = true;
				}
				else {
					StringTokenizer tokenizer = new StringTokenizer(args[n + 1], ":");
					int             portCount = tokenizer.countTokens();
					try {
						if(portCount == 1) {
							ftpDataPortFrom = Integer.parseInt(tokenizer.nextToken());
							ftpDataPortTo   = ftpDataPortFrom;
						}
						else if(portCount == 2) {
							ftpDataPortFrom = Integer.parseInt(tokenizer.nextToken());
							ftpDataPortTo   = Integer.parseInt(tokenizer.nextToken());
						}
						else {
							System.err.println("Need to specify two integer numbers : " + args[n + 1]);
							showUsage = true;
						}
					}
					catch(NumberFormatException e) {
						System.err.println("Need to specify two integer numbers : " + args[n + 1]);
						showUsage = true;
					}
					n++;
				}
			}
			else if(args[n].equals("-d")) {
				if(n + 1 >= args.length || args[n + 1].startsWith("-")) {
					System.err.println("Missing a directory : " + args[n]);
					showUsage = true;
				}
				else {
					File directory = new File(args[n + 1]);
					if(directory.isDirectory()) {
						dstDirectory = args[n + 1];
					}
					else {
						System.err.println("Need to specify a directory path : " + args[n + 1]);
						showUsage = true;
					}
					n++;
				}
			}
			else if(args[n].equals("-b")) {
				if(n + 1 >= args.length || args[n + 1].startsWith("-")) {
					System.err.println("Missing a buffer size : " + args[n]);
					showUsage = true;
				}
				else {
					try {
						ftpBufferSize = Integer.parseInt(args[n + 1]);
					}
					catch(NumberFormatException e) {
						System.err.println("Need to specify an integer numbers : " + args[n + 1]);
						showUsage = true;
					}
					n++;
				}
			}
			else if(args[n].equals("-n")) {
				if(n + 1 >= args.length || args[n + 1].startsWith("-")) {
					System.err.println("Missing an NETBIOS name : " + args[n]);
					showUsage = true;
				}
				else {
					serviceName = args[n + 1];
					n++;
				}
			}
			else if(args[n].equals("-s")) {
				showSummary = true;
			}
			else if(args[n].equals("-h")) {
				showUsage = true;
				showHelp  = true;
			}
			else {
				System.err.println("Unknown option : " + args[n]);
				showUsage = true;
			}
		}

		// コマンドの引数を処理する。
		if(showUsage) {
			System.err.println("Usage : " + kProgramName + " [-p <from>:<to>] [-b <size>] [-d <directory>] [-n <name>] [-c] [-s] [-h]");
		}
		if(showHelp) {
			System.err.println("Options :");
			System.err.println("  -p : Specify the range of the data port for the FTP connection.");
			System.err.println("  -b : Specify the FTP buffer size.");
			System.err.println("  -d : Specify the destination directory.");
			System.err.println("  -n : Specify the NETBIOS name.");
			System.err.println("  -s : Display the setting summaries.");
			System.err.println("  -h : Display the help messages.");
		}
		if(showHelp || showUsage) {
			System.exit(0);
		}
		if(ftpDataPortFrom == 0) {
			ftpDataPortTo = 0;
		}
		fFTPDataPortFrom = ftpDataPortFrom;
		fFTPDataPortTo   = ftpDataPortTo;
		fFTPBufferSize   = ftpBufferSize;
		fDstDirectory    = dstDirectory;
		fServiceName     = serviceName;
		
		// オープニング・メッセージを表示する。
		System.out.println(kProgramName + " " + kProgramVersion);
		System.out.println("Copyright 2004,2005 raktajino. (http://homepage.mac.com/raktajino/)");
		if(showSummary) {
			System.out.println("FTP control           : port " + fFTPControlPort);
			if(fFTPDataPortFrom == 0) {
				System.out.println("FTP data              : (automatic assignment)");
			}
			else {
				System.out.println("FTP data              : port " + fFTPDataPortFrom + " - " + fFTPDataPortTo);
			}
			System.out.println("FTP buffer size       : " + fFTPBufferSize);
			System.out.println("NETBIOS NS            : port " + fNetBIOSNSPort);
			System.out.println("NETBIOS service name  : " + fServiceName);
			System.out.println("Destination directory : " + fDstDirectory);
		}
		System.out.println("Ready.");
	}
}
